<?PHP
    //This php script contains all the stuff to backup/restore
    //teacherplan mods

    //This is the "graphical" structure of the teacherplan mod:
    //
    //        teacherplan      teacherplan_objectives
    //        (CL,pk->id)          (CL, pk->id)
    //             |                    | 
    //             |____________________|
    //                       |
    //            teacherplan_objectives_met 
    //     (pk->id, fk->(planid, objectiveid), course)
    //
    // Meaning: pk->primary key field of the table
    //          fk->foreign key to link with parent
    //          nt->nested field (recursive data)
    //          CL->course level info
    //          UL->user level info
    //
    //-----------------------------------------------------------

//This function executes all the backup procedure about this mod
function teacherplan_backup_mods($bf,$preferences) {
    global $CFG;

    $status = true;

    $status = ($status and teacherplan_backup_plans($bf, $preferences));
    $status = ($status and teacherplan_backup_objectives($bf, $preferences));
    $status = ($status and teacherplan_backup_standards($bf, $preferences));
    $status = ($status and teacherplan_backup_objectives_met($bf, $preferences));

    return $status;
}

function teacherplan_backup_plans($bf, $preferences) {
    //Iterate over teacherplan table
    $plans = get_records ("teacherplan","course",$preferences->backup_course,"id");
    if ($plans) {
        foreach ($plans as $plan) {
            //Start mod
            fwrite ($bf,start_tag("MOD",3,true));
            //Print teacherplan data
            fwrite ($bf,full_tag("ID",4,false,$plan->id));
            fwrite ($bf,full_tag("MODTYPE",4,false,"teacherplan"));
            fwrite ($bf,full_tag("PLANSUBTABLE",4,false,"teacherplan"));
            fwrite ($bf,full_tag("NAME",4,false,$plan->name));
            fwrite ($bf,full_tag("PLAN",4,false,$plan->plan));
            fwrite ($bf,full_tag("ASSESSMENT",4,false,$plan->assessment));
            fwrite ($bf,full_tag("MATERIALS",4,false,$plan->materials));
            fwrite ($bf,full_tag("PERIOD",4,false,$plan->period));
            fwrite ($bf,full_tag("PLANDATE",4,false,$plan->plandate));
            fwrite ($bf,full_tag("MULTIDAY",1,false,$plan->multiday));
            fwrite ($bf,full_tag("PLANENDDATE",4,false,$plan->planenddate));
            fwrite ($bf,full_tag("TIMEMODIFIED",4,false,$plan->timemodified));

            //if we've selected to backup users info, then execute backup_assignment_submisions
//            if ($preferences->mods["plan"]->userinfo) {
//            }
            //End mod
            $status = fwrite ($bf,end_tag("MOD",3,true));
        }
    }

    return $status;
}


function teacherplan_backup_objectives($bf, $preferences) {
    //Iterate over objectives table
    $objectives = get_records ("teacherplan_objectives","course",$preferences->backup_course,"id");
    if ($objectives) {
        foreach ($objectives as $objective) {
            //Start mod
            fwrite ($bf,start_tag("MOD",3,true));
            //Print objectives data
            fwrite ($bf,full_tag("ID",4,false,$objective->id));
            fwrite ($bf,full_tag("MODTYPE",4,false,"teacherplan"));
            fwrite ($bf,full_tag("PLANSUBTABLE",4,false,"teacherplan_objectives"));
            fwrite ($bf,full_tag("NAME",4,false,$objective->name));
            fwrite ($bf,full_tag("OBJECTIVE",4,false,$objective->objective));
            fwrite ($bf,full_tag("STANDARDID",4,false,$objective->standardid));
            fwrite ($bf,full_tag("TIMEMODIFIED",4,false,$objective->timemodified));

            //if we've selected to backup users info, then execute backup_assignment_submisions
//            if ($preferences->mods["plan"]->userinfo) {
//            }
            //End mod
            $status = fwrite ($bf,end_tag("MOD",3,true));
        }
    }

    return $status;
}

function teacherplan_backup_objectives_met($bf, $preferences) {
    //Iterate over objectives_met table
    $objectives_met = get_records ("teacherplan_objectives_met","course",$preferences->backup_course,"id");
    if ($objectives_met) {
        foreach ($objectives_met as $objective_met) {
            //Start mod
            fwrite ($bf,start_tag("MOD",3,true));
            //Print objective_met data
            fwrite ($bf,full_tag("ID",4,false,$objective_met->id));
            fwrite ($bf,full_tag("MODTYPE",4,false,"teacherplan"));
            fwrite ($bf,full_tag("PLANSUBTABLE",4,false,"teacherplan_objectives_met"));
            fwrite ($bf,full_tag("PLANID",4,false,$objective_met->planid));
            fwrite ($bf,full_tag("OBJECTIVEID",4,false,$objective_met->objectiveid));
            fwrite ($bf,full_tag("TIMEMODIFIED",4,false,$objective_met->timemodified));

            //if we've selected to backup users info, then execute backup_assignment_submisions
//            if ($preferences->mods["plan"]->userinfo) {
//            }
            //End mod
            $status = fwrite ($bf,end_tag("MOD",3,true));
        }
    }

    return $status;
}

function teacherplan_backup_standards($bf, $preferences) {
    //Iterate over objectives table
    $standards = get_records ("teacherplan_standards","course",$preferences->backup_course,"id");
    if ($standards) {
        foreach ($standards as $standard) {
            //Start mod
            fwrite ($bf,start_tag("MOD",3,true));
            //Print objectives data
            fwrite ($bf,full_tag("ID",4,false,$standard->id));
            fwrite ($bf,full_tag("MODTYPE",4,false,"teacherplan"));
            fwrite ($bf,full_tag("PLANSUBTABLE",4,false,"teacherplan_standards"));
            fwrite ($bf,full_tag("NAME",4,false,$standard->name));
            fwrite ($bf,full_tag("STANDARD",4,false,$standard->standard));
            fwrite ($bf,full_tag("PARENTID",4,false,(empty($standard->parentid) ? -1 : $standard->parentid)));
            fwrite ($bf,full_tag("TIMEMODIFIED",4,false,$standard->timemodified));

            //if we've selected to backup users info, then execute backup_assignment_submisions
//            if ($preferences->mods["plan"]->userinfo) {
//            }
            //End mod
            $status = fwrite ($bf,end_tag("MOD",3,true));
        }
    }

    return $status;
}

//Return an array of info (name,value)
function teacherplan_check_backup_mods($course,$user_data=false,$backup_unique_code) {
    //First the course data
    $info[0][0] = get_string('modulenameplural','teacherplan');
    if ($ids = teacherplan_ids ($course)) {
        $info[0][1] = count($ids);
    } else {
        $info[0][1] = 0;
    }

    $info[1][0] = get_string('objectivestexttitle','teacherplan');
    if ($ids = teacherplan_objective_ids ($course)) {
        $info[1][1] = count($ids);
    } else {
        $info[1][1] = 0;
    }

    $info[1][0] = get_string('standardtitle','teacherplan');
    if ($ids = teacherplan_standard_ids ($course)) {
        $info[1][1] = count($ids);
    } else {
        $info[1][1] = 0;
    }

    return $info;
}

//Return a content encoded to support interactivities linking. Every module
//should have its own. They are called automatically from the backup procedure.
function teacherplan_encode_content_links ($content,$preferences) {

    global $CFG;

    $base = preg_quote($CFG->wwwroot,"/");

     //Link to the list of teacherplans
    $buscar="/(".$base."\/mod\/teacherplan\/index.php\?id\=)([0-9]+)/";
    $result= preg_replace($buscar,'$@TEACHERPLANINDEX*$2@$',$content);

    //Link to teacherplan view by moduleid
    $buscar="/(".$base."\/mod\/teacherplan\/view.php\?id\=)([0-9]+)/";
    $result= preg_replace($buscar,'$@TEACHERPLANVIEWBYID*$2@$',$result);

    return $result;
}

    // INTERNAL FUNCTIONS. BASED IN THE MOD STRUCTURE

    //Returns an array of teacherplan id 
    function teacherplan_ids ($course) {

        global $CFG;

        return get_records_sql ("SELECT a.id, a.course
                                 FROM {$CFG->prefix}teacherplan a
                                 WHERE a.course = '$course'");
    }
    
    //Returns an array of teacherplan_objective ids 
    function teacherplan_objective_ids ($course) {

        global $CFG;

        return get_records_sql ("SELECT a.id, a.course
                                 FROM {$CFG->prefix}teacherplan_objectives a
                                 WHERE a.course = '$course'");
    }

    //Returns an array of teacherplan_standard ids 
    function teacherplan_standard_ids ($course) {

        global $CFG;

        return get_records_sql ("SELECT a.id, a.course
                                 FROM {$CFG->prefix}teacherplan_standards a
                                 WHERE a.course = '$course'");
    }
?>