<?php // $Id: objectives.php,v 1.5 2006/07/27 08:14:15 scarletpumpkin Exp $
/**
* Functions to handle working with objectives
*/
global $CFG;
require_once("$CFG->dirroot/lib/datalib.php");

if (!isset($CFG->teacherplan_allowobjedit)) {
    set_config('teacherplan_allowobjedit', 'yes');
}

/**
* Get objectives from objective-plan table and return list of objecives used in 
*  given plan as an array of strings
*
* @param tpid      integer, teacherplan ID
* @param courseid  integer, course id
* @return   array of strings containing objectives|false if none found
*/
function teacherplan_get_objectives_string($tpid=0, $courseid=0){

    unset($objectivestrings);
    if ($objectives = teacherplan_get_objectives_array($tpid, $courseid)) {
        // Make record list into list of strings only
        foreach ($objectives as $objective) {
            $objectivestrings[] = $objective->objective;    
        }
    }
  
    return $objectivestrings;
}


/**
* Get a list of benchmarks with corresponding standards (including designations)
*
* @param courseid   integer, course id
* @param tpid       integer, teacherplan ID
*/
function get_standards_benchmarks_array($tpid=-1, $courseid=0){

    if (!isset($CFG->teacherplan_designationseparator)) {
        $CFG->teacherplan_designationseparator = '-';
    }

    $sep = $CFG->teacherplan_designationseparator;
    
    if ($list = teacherplan_get_objectives_array($tpid, $courseid)) {
        foreach ($list as $item) {
            unset($data);
            if ($standard = get_record('teacherplan_standards', 'id', $item->standardid)) {
                $data->standard = $standard->standard;
                $data->standarddesc = $standard->name;                    
                $data->standardid = $standard->id; 
                $data->standardparent = (empty($standard->parentid) ? -1: $standard->parentid); 
                $compdesc = $standard->name.$sep.$item->name;
                $parentid = $data->standardparent;
                while ($parentid !== -1) {
                    if ($parent = get_record('teacherplan_standards', 'id', $parentid)) {
                        $parentid = (empty($parent->parentid) ? -1: $parent->parentid);
                        $compdesc = $parent->name.$sep.$compdesc;
                    }
                } 
                $data->completedesc = $compdesc;                 
            } else {
                $data->standard = '';
                $data->standarddesc = '';                    
                $data->standardid = -1;                                
                $data->standardparent = -1;      
                $data->standardcompletedesc = '';             
            }
            $data->objective = $item->objective; 
            $data->objectiveid = $item->id;             
            $data->objectivedesc = $item->name;   
            $returnarray[] = $data;   
        }
    }
  
    return (isset($returnarray) ? $returnarray : NULL);
}

/**
* Get standards tree from table
*
* @param courseid  integer, course id
* @return   linked list of standards
*/
function &get_linked_standards_list($courseid) {
  //for non-linked standards
    $node->parentid = -1;
    $node->parent = NULL;
    $node->children = NULL;
    $node->text = get_string('generalbenchmarkslabel', 'teacherplan');
    $node->designation = '';
    $node->id = -1;
    $node->printme = false;
    $tree[-1] =& $node;
    
    //for all others
    if ($standards = get_records('teacherplan_standards', 'course', $courseid, 'parentid ASC,name ASC')) {
        foreach ($standards as $standard) {
            unset($node);
            $node->parentid = (empty($standard->parentid) ? -1: $standard->parentid);
            $node->parent = NULL;
            $node->children = NULL;
            $node->text = $standard->standard;
            $node->designation = $standard->name;
            $node->id = $standard->id;
            $node->printme = false;
            $tree[$standard->id] =& $node;
        }
        
        //link parents and children
        foreach ($tree as $key => $value) {   
            $parentid = $tree[$key]->parentid;  
            if (isset($tree[$parentid]) and $parentid !== -1) {
                $tree[$key]->parent =& $tree[$parentid];
                $num = count($tree[$parentid]->children);
                $tree[$parentid]->children[$num] =& $tree[$key];
            }
        }          
    }     

    return $tree;
}

/**
* Get objectives from objective-plan table and return list of objecives used in 
*  given plan as an associative array of (objectiveid => string)
*
* @param tpid      integer, teacherplan ID
* @param courseid  integer, course id
* @return   associative array of strings containing objectives|false if none found
*/
function teacherplan_get_objectives_array($tpid=0, $courseid=0){
    //If no teacherplan id given (default set), return false
    if ($tpid == 0) {
        return false;
    //If teacherplan id < 0 then only look at courseid, getting a list of all objectives in this course
    } else if ($tpid < 0) {
        $objectives = get_records("teacherplan_objectives", "course", $courseid, 'standardid,name,objective');
        if (!empty($objectives)) {
            return $objectives;
        }
    //Use teacherplanid to get all objectives associated with this plan
    } else {
        $objectives = get_records('teacherplan_objectives_met', 'planid', $tpid, '', 'objectiveid,planid');

        // Get list of objectiveid,planid pairs.  Notice that the planid should be the same for all
        if (empty($objectives)) {
            return false;
        }

        // Get list of objectives from table
        $objectivelist = '';
        foreach ($objectives as $objective) {
            if (!empty($objectivelist)) {
                $objectivelist = $objectivelist.",";
            }
            $objectivelist = $objectivelist.$objective->objectiveid;
        }

        $objectives = get_records_list("teacherplan_objectives", "id", $objectivelist, 'standardid,name,objective');
        if (!empty($objectives)) {
            return $objectives;
        }
    }


    return false;
}

/**
* Print a select box for selecting objectives
*
* @param courseid   integer, course id
* @param tpid       integer, teacherplan ID
* @param $fieldname string,  fieldname for select box     
* @param $multiple  boolean, if true, make selection multiple
* @param $onChange  string,  function to call when select changes
* @param $style     string,  CSS style specifics
* @param $size      integer, number of lines to display
* @param $useseparator      boolean, true to display separator, false otherwise
*/
function print_objectives_selectbox($courseid=0, $tpid=0, $fieldname='', $multiple=false, $onChange='', $style='width: 600px', $size=10, $useseparator=true) {

    if (!isset($CFG->teacherplan_designationseparator)) {
        $CFG->teacherplan_designationseparator = '-';
    }

    $sep = $CFG->teacherplan_designationseparator;
    $onChngText = empty($onChange)?'':' onChange="'.$onChange.'" ';
    $fieldNameTxt = $multiple?$fieldname.'[]':$fieldname;
    echo '<select style="'.$style.'"';
    if ($multiple) {
       echo ' multiple ';
    }
    echo ' size="'.$size.'" name="'.$fieldNameTxt.'"'.$onChngText.'>';

    $allobjectives = get_standards_benchmarks_array(-1, $courseid);
    if ($tpid == 0) {
        $objectivesused = '';
    } else {
        $objectivesused = get_standards_benchmarks_array($tpid, $courseid);
    }

    if ($allobjectives) {
        foreach ($allobjectives as $objective) {
            echo "<option ";
            if (!empty($objectivesused)) {
                foreach ($objectivesused as $objectiveused) {
                    if ($objectiveused->objectiveid == $objective->objectiveid) {
                        echo "selected ";
                        break;
                    }
                }
            }
            echo 'value="'.$objective->objectiveid.'">';
            if ($useseparator) {
                echo $objective->completedesc.$sep;
            }
            echo $objective->objective.'</option>';            
        }
   }
   echo "</select>";
}

/**
* Print a link to edit objectives for a course
*
* @param courseid   integer, course id
* @param $formname  string,  objective selectbox name
* @param $reactfunc string,  JavaScript function to react to changes in $formname (of form reactfunc())   
*/
function teacherplan_editobjectives_link($courseid, $formname='', $reactfunc='') {
    global $CFG;
    if ($CFG->teacherplan_allowobjedit == "no") {
        return "";
    }
    
    if (!empty($formname)) {
        $formname = '&parent='.$formname;
    }  
    if (!empty($reactfunc)) {
        $reactfunc = '&react='.$reactfunc;
    }  
    echo "<a title=\"Objectives\" target=\"objectivewindow\" onclick=\"return openpopup('/mod/teacherplan/configtp.php?courseid=$courseid".$formname.$reactfunc."', 'objectivewindow','height=620,width=620,resizable=yes', false);\" href=\"$CFG->wwwroot/mod/teacherplan/configtp.php?courseid=$courseid".$formname.$reactfunc."\">".get_string('objectivesblocktitle', 'teacherplan')."</a>";
}

/**
* Return a link for viewing objectives
*
* @param courseid   integer, course id
* @param tpid       integer, plan id
*/
function teacherplan_viewobjectives_link($courseid, $tpid=-1) {
    global $CFG;  
    
    return "<a title=\"Objectives\" target=\"objectivewindow\" onclick=\"return openpopup('/mod/teacherplan/viewobjectives.php?id=$courseid&tpid=$tpid', 'objectivewindow','height=400,width=620,scrollbars=1,resizable=1', false);\" href=\"$CFG->wwwroot/mod/teacherplan/viewobjectives.php?id=$courseid&tpid=$tpid\">".get_string('viewobjectivestitle', 'teacherplan')."</a>";
}

/**
* Print formatted list of standards and benchmark for the course
*  If the plan id is given, then the list is restricted to those
*  used in that plan.
*
* @param courseid   integer, course id
* @param tpid       integer, plan id
* @param link       boolean, provide bookmark to list items, if true
* @param callback  string, name of function to pass benchmark id to after printing benchmark  
*/
function teacherplan_get_pretty_objectives($tpid=-1, $courseid=0, $link=false, $callback='') {
  
    $retval = '';
    $standardtree =& get_linked_standards_list($courseid);
    
    if ($list = get_standards_benchmarks_array($tpid, $courseid)) {
        foreach ($list as $item) {
            if (!isset($standards[$item->standardid])) {
                unset($lastnode);
                $keeprunning = true;
                do {
                    if (isset($lastnode)) {
                        $parentid = $standardtree[$lastnode]->parentid;
                        $stext = $standardtree[$lastnode]->text;
                        $sdesig = $standardtree[$lastnode]->designation;
                        $sid = $standardtree[$lastnode]->id;
                    } elseif ($item->standardid == -1) {
                        $parentid = -1;
                        $stext = $standardtree[-1]->text;
                        $sdesig = $standardtree[-1]->designation;
                        $sid = -1;
                    } else {
                        $parentid = $item->standardparent;
                        $stext = $item->standard;
                        $sdesig = $item->standarddesc;
                        $sid = $item->standardid;
                    }
                                        
                    if (!isset($standards[$sid])) {
                        $printtext = '<b>'.$sdesig.'</b>&nbsp;'.$stext;           
                        $printtext = format_text($printtext, FORMAT_MOODLE, NULL, $courseid);
                        if ($link) {
                            $printtext = '<a name="s'.$sid.'">'.$printtext.'</a>';
                        }
                        $standards[$sid] = $printtext;
                        $standardtree[$sid]->printme = true;
                    }
                    
                    if ($parentid !== -1) {
                        $lastnode = $parentid; 
                    } else {
                        if (array_search($sid, $headstandards) === false) {
                            $headstandards[] = $sid;
                        }
                        $keeprunning = false;     
                    }
                } while ($keeprunning);   
            }
            
            $printtext = '<b>'.$item->objectivedesc.'</b>&nbsp;'.$item->objective;
            $printtext = format_text($printtext, FORMAT_MOODLE, NULL, $courseid);
            if ($link) {
                $printtext = '<a name="b'.$item->objectiveid.'">'.$printtext.'</a>';
            }
            $bindex = $item->objectivedesc.' '.$item->objectiveid;
            $benches[$item->standardid][$bindex]->text = $printtext;
            $benches[$item->standardid][$bindex]->id = $item->objectiveid;
        }
        
        foreach ($standards as $skey => $standard) {
            ksort($benches[$skey]);
            reset($benches[$skey]);
        }
                
        foreach ($headstandards as $key => $value) {
            $retval .= teacherplan_print_standards($standardtree[$value], $standards, $benches, $callback);
        }  
    }
    
    return $retval;
}

function teacherplan_print_standards(&$treeitem, &$standards, &$benches, $callback) {
    $retval = '';
    if ($treeitem->parentid !== -1) {
        $retval .= '<li>';
    }
    
    $retval .= $standards[$treeitem->id].'<ul>';
    if (isset($benches[$treeitem->id])) {
        foreach ($benches[$treeitem->id] as $bkey => $benchmark) {
            $retval .= '<li>'.$benchmark->text;
            if (!empty($callback)) {
                $retval .= call_user_func($callback, $benchmark->id);
            }
        }
    }

    if (!empty($treeitem->children)){
        for ($i = 0; $i < count($treeitem->children); $i++) {
            if ($treeitem->children[$i]->printme) {
                $retval .= teacherplan_print_standards($treeitem->children[$i], $standards, $benches, $callback);
            }
        }
    }           

    $retval .= '</ul>';

    return $retval;
}
?>