<?PHP
    //This php script contains all the stuff to backup/restore
    //teacherplan mods.
    // Much of the code was heavily borrowed from the assignment
    // module restore code of version 1.5.3

    //This is the "graphical" structure of the teacherplan mod:
    //
    //        teacherplan      teacherplan_objectives
    //        (CL,pk->id)          (CL, pk->id)
    //             |                    | 
    //             |____________________|_________________________
    //                       |                                    | 
    //            teacherplan_objectives_met              teacherplan_standards 
    //     (pk->id, fk->(planid, objectiveid), course)    (pk->id, fk->(standardid), course)
    //
    // Meaning: pk->primary key field of the table
    //          fk->foreign key to link with parent
    //          nt->nested field (recursive data)
    //          CL->course level info
    //          UL->user level info
    //
    //-----------------------------------------------------------

//This function executes all the restore procedure about this mod
function teacherplan_restore_mods($mod,$restore) {
    global $CFG;

    $status = true;

    //Get record from backup_ids
    $data = backup_getid($restore->backup_unique_code,$mod->modtype,$mod->id);

    if ($data) {
        //Now get completed xmlized object   
        $info = $data->info;
        //traverse_xmlize($info);                                                                     //Debug
        //print_object ($GLOBALS['traverse_array']);                                                  //Debug
        //$GLOBALS['traverse_array']="";                                                              //Debug
        $subtype = backup_todb($info['MOD']['#']['PLANSUBTABLE']['0']['#']);
                
        switch ($subtype) {
            case 'teacherplan':

                //Now, build the TEACHERPLAN record structure
                $teacherplan->course = $restore->course_id;
                $teacherplan->name = backup_todb($info['MOD']['#']['NAME']['0']['#']);
                $teacherplan->plan = backup_todb($info['MOD']['#']['PLAN']['0']['#']);
                $teacherplan->assessment = backup_todb($info['MOD']['#']['ASSESSMENT']['0']['#']);
                $teacherplan->materials = backup_todb($info['MOD']['#']['MATERIALS']['0']['#']);
                $teacherplan->period = backup_todb($info['MOD']['#']['PERIOD']['0']['#']);
                $teacherplan->plandate = backup_todb($info['MOD']['#']['PLANDATE']['0']['#']);
                $teacherplan->multiday = backup_todb($info['MOD']['#']['MULTIDAY']['0']['#']);
                $teacherplan->planenddate = backup_todb($info['MOD']['#']['PLANENDDATE']['0']['#']);
                $teacherplan->timemodified = backup_todb($info['MOD']['#']['TIMEMODIFIED']['0']['#']);

                //The structure is equal to the db, so insert the teacherplan
                $newid = insert_record ("teacherplan", $teacherplan);

                //Do some output     
                echo "<li>".get_string("modulename","teacherplan")." \"".format_string(stripslashes($teacherplan->name),true)."\"</li>";

                break;
            case 'teacherplan_objectives':

                //Now, build the OBJECTIVES record structure
                $objectives->course = $restore->course_id;
                $objectives->name = backup_todb($info['MOD']['#']['NAME']['0']['#']);
                $objectives->objective = backup_todb($info['MOD']['#']['OBJECTIVE']['0']['#']);
                $objectives->standardid = backup_todb($info['MOD']['#']['STANDARDID']['0']['#']);
                $objectives->timemodified = backup_todb($info['MOD']['#']['TIMEMODIFIED']['0']['#']);

                //The structure is equal to the db, so insert the teacherplan
                $newid = insert_record ("teacherplan_objectives", $objectives);

                //Do some output     
                if ($newid) {
                    echo "<li>".get_string("objectivestexttitle","teacherplan").":".$objectives->objective."</li>";
                }

                break;

            case 'teacherplan_standards':

                //Now, build the STANDARDS record structure
                $standard->course = $restore->course_id;
                $standard->name = backup_todb($info['MOD']['#']['NAME']['0']['#']);
                $standard->standard = backup_todb($info['MOD']['#']['STANDARD']['0']['#']);
                $temp = backup_todb($info['MOD']['#']['PARENTID']['0']['#']);
                if ($temp !== -1) {//leave NULL otherwise
                    $standard->parentid = $temp;
                }
                $standard->timemodified = backup_todb($info['MOD']['#']['TIMEMODIFIED']['0']['#']);

                //The structure is equal to the db, so insert the standard
                $newid = insert_record ("teacherplan_standards", $standard);

                //Do some output     
                if ($newid) {
                    echo "<li>".get_string("standardtitle","teacherplan").":".$standard->name."</li>";
                }

                break;

            case 'teacherplan_objectives_met':

                //Now, build the OBJECTIVES_MET record structure
                $objective_met->course = $restore->course_id;
                $objective_met->planid = backup_todb($info['MOD']['#']['PLANID']['0']['#']);
                $objective_met->objectiveid = backup_todb($info['MOD']['#']['OBJECTIVEID']['0']['#']);
                $objective_met->timemodified = backup_todb($info['MOD']['#']['TIMEMODIFIED']['0']['#']);

                //The structure is equal to the db, so insert the teacherplan
                if ($objective_met->planid == 0 || $objective_met->objectiveid == 0) {
                   $newid = -1;
                } else {
                   $newid = insert_record ("teacherplan_objectives_met", $objective_met);
                }

                //Do some output     
                if ($newid) {
                    echo "<li>Objective met added:(".$objective_met->planid.",".$objective_met->objectiveid.")</li>";
                }

                break;
            default:
                break;
        }

        backup_flush(300);
        if ($newid == -1) {
            $status = true;
        } elseif ($newid) {
            //We have the newid, update backup_ids
            backup_putid($restore->backup_unique_code,$mod->modtype,
                         $mod->id, $newid);
        } else {
            $status = false;
        }
    } else {
        $status = false;
    }

    return $status;
}

//Return a content decoded to support interactivities linking. Every module
//should have its own. They are called automatically from
//teacherplan_decode_content_links_caller() function in each module
//in the restore process
function teacherplan_decode_content_links ($content,$restore) {
          
    global $CFG;
           
    $result = $content;
                
    //Link to the list of teacherplan
                
    $searchstring='/\$@(TEACHERPLANINDEX)\*([0-9]+)@\$/';
    //We look for it
    preg_match_all($searchstring,$content,$foundset);
    //If found, then we are going to look for its new id (in backup tables)
    if ($foundset[0]) {
        //print_object($foundset);                                     //Debug
        //Iterate over foundset[2]. They are the old_ids
        foreach($foundset[2] as $old_id) {
            //We get the needed variables here (course id)
            $rec = backup_getid($restore->backup_unique_code,"course",$old_id);
            //Personalize the searchstring
            $searchstring='/\$@(TEACHERPLANINDEX)\*('.$old_id.')@\$/';
            //If it is a link to this course, update the link to its new location
            if($rec->new_id) {
                //Now replace it
                $result= preg_replace($searchstring,$CFG->wwwroot.'/mod/teacherplan/index.php?id='.$rec->new_id,$result);
            } else { 
                //It's a foreign link so leave it as original
                $result= preg_replace($searchstring,$restore->original_wwwroot.'/mod/teacherplan/index.php?id='.$old_id,$result);
            }
        }
    }

    //Link to teacherplan view by moduleid

    $searchstring='/\$@(TEACHERPLANBYID)\*([0-9]+)@\$/';
    //We look for it
    preg_match_all($searchstring,$result,$foundset);
    //If found, then we are going to look for its new id (in backup tables)
    if ($foundset[0]) {
        //print_object($foundset);                                     //Debug
        //Iterate over foundset[2]. They are the old_ids
        foreach($foundset[2] as $old_id) {
            //We get the needed variables here (course_modules id)
            $rec = backup_getid($restore->backup_unique_code,"course_modules",$old_id);
            //Personalize the searchstring
            $searchstring='/\$@(TEACHERPLANBYID)\*('.$old_id.')@\$/';
            //If it is a link to this course, update the link to its new location
            if($rec->new_id) {
                //Now replace it
                $result= preg_replace($searchstring,$CFG->wwwroot.'/mod/teacherplan/view.php?id='.$rec->new_id,$result);
            } else {
                //It's a foreign link so leave it as original
                $result= preg_replace($searchstring,$restore->original_wwwroot.'/mod/teacherplan/view.php?id='.$old_id,$result);
            }
        }
    }

    return $result;
}

//This function makes all the necessary calls to xxxx_decode_content_links()
//function in each module, passing them the desired contents to be decoded
//from backup format to destination site/course in order to mantain inter-activities
//working in the backup/restore process. It's called from restore_decode_content_links()
//function in restore process
function teacherplan_decode_content_links_caller($restore) {
    global $CFG;

    $status = true;

    $status = teacherplan_recode_met_ids($restore);
    
    $status = ($status and teacherplan_recode_standard_ids($restore));

    $status = ($status and teacherplan_recode_standard_parent_ids($restore));

    if ($teacherplans = get_records_sql ("SELECT a.id, a.plan
                               FROM {$CFG->prefix}teacherplan a
                               WHERE a.course = $restore->course_id")) {
        //Iterate over each teacherplan->plan
        $i = 0;   //Counter to send some output to the browser to avoid timeouts
        foreach ($teacherplans as $teacherplan) {
            //Increment counter
            $i++;
            $content = $teacherplan->plan;
            $result = restore_decode_content_links_worker($content,$restore);
            if ($result != $content) {
                //Update record
                $teacherplan->description = addslashes($result);
                $status = update_record("teacherplan",$teacherplan);
                if ($CFG->debug>7) {
                    echo '<br /><hr />'.htmlentities($content).'<br />changed to<br />'.htmlentities($result).'<hr /><br />';
                }
            }
            //Do some output
            if (($i+1) % 5 == 0) {
                echo ".";
                if (($i+1) % 100 == 0) {
                    echo "<br />";
                }
                backup_flush(300);
            }
        }
    }

    return $status;
}

function teacherplan_recode_met_ids($restore) {
    global $CFG;
 
    $status = true;

    if ($objectives_met = get_records_sql ("SELECT a.id, a.objectiveid, a.planid
                                 FROM {$CFG->prefix}teacherplan_objectives_met a
                                 WHERE a.course = $restore->course_id")) {
        //Iterate over each teacherplan->plan
        $i = 0;   //Counter to send some output to the browser to avoid timeouts
        foreach ($objectives_met as $met) {
            if ($objectiverec = backup_getid($restore->backup_unique_code, 'teacherplan_objectives',$met->objectiveid)) {
                $met->objectiveid = $objectiverec->id;
            }

            if ($planrec = backup_getid($restore->backup_unique_code, 'teacherplan',$met->planid)) {
                $met->planid = $planrec->id;
            }

            if ($planrec and $objectiverec) {
                $statustemp = update_record('teacherplan_objectives_met', $met);
                $status = ($status and $statustemp); 
            }

            //Do some output
            if (($i+1) % 5 == 0) {
                echo ".";
                if (($i+1) % 100 == 0) {
                    echo "<br/>";
                }
                backup_flush(300);
            }
            $i++;
        }
    }

    return $status;
}

function teacherplan_recode_standard_ids($restore) {
    global $CFG;
 
    $status = true;

    if ($standard_ids = get_records_sql ("SELECT a.id, a.standardid
                                 FROM {$CFG->prefix}teacherplan_objectives a
                                 WHERE a.course = $restore->course_id")) {
        //Iterate over each objective
        $i = 0;   //Counter to send some output to the browser to avoid timeouts
        foreach ($standard_ids as $met) {
            if ($standardrec = backup_getid($restore->backup_unique_code, 'teacherplan_standards',$met->standardid)) {
                $met->standardid = $standardrec->id;
            }

            if ($standardrec) {
                $statustemp = update_record('teacherplan_objectives', $met);
                $status = ($status and $statustemp);
            }

            //Do some output
            if (($i+1) % 5 == 0) {
                echo ".";
                if (($i+1) % 100 == 0) {
                    echo "<br/>";
                }
                backup_flush(300);
            }
            $i++;
        }
    }

    return $status;
}

function teacherplan_recode_standard_parent_ids($restore) {
    global $CFG;
 
    $status = true;

    if ($standard_ids = get_records_sql ("SELECT a.id, a.parentid
                                 FROM {$CFG->prefix}teacherplan_standards a
                                 WHERE a.course = $restore->course_id")) {
        //Iterate over each objective
        $i = 0;   //Counter to send some output to the browser to avoid timeouts
        foreach ($standard_ids as $met) {
            if ($standardrec = backup_getid($restore->backup_unique_code, 'teacherplan_standards',$met->parentid)) {
                $met->parentid = $standardrec->id;
            }

            if ($standardrec) {
                $statustemp = update_record('teacherplan_standards', $met);
                $status = ($status and $statustemp);
            }

            //Do some output
            if (($i+1) % 5 == 0) {
                echo ".";
                if (($i+1) % 100 == 0) {
                    echo "<br/>";
                }
                backup_flush(300);
            }
            $i++;
        }
    }

    return $status;
}
function teacherplan_restore_logs($restore, $logs) {
    return true;
}

?>